using System;
using System.Collections.Generic;
using System.Text;
using FooCfgToXml;
using System.IO;
using System.Xml;

namespace UiColumnsInterpreters
{
	public static class Utilities
	{
		/// <returns>The interpreter for this item, if any</returns>
		public static IItemInterpreter WriteItemTypeToXml(XmlWriter writer, Stream reader, Dictionary<Guid, IItemInterpreter> itemInterpreters)
		{
			return WriteItemTypeToXml(writer, CfgToXmlUtilities.ReadGuid(reader), itemInterpreters);
		}

		/// <summary>This overload is used to delay writing out the item type from the time the guid is read.</summary>
		/// <returns>The interpreter for this item, if any</returns>
		public static IItemInterpreter WriteItemTypeToXml(XmlWriter writer, Guid itemType, Dictionary<Guid, IItemInterpreter> itemInterpreters)
		{
			string itemTypeName;
			//See if there is an interpreter for this value
			IItemInterpreter itemInterpreter;
			if (itemInterpreters.TryGetValue(itemType, out itemInterpreter))
			{
				itemTypeName = itemInterpreter.TypeName;
				writer.WriteAttributeString("interpreter", itemInterpreter.GetType().FullName);				
			}
			else
			{
				//No interpreter, so just use the guid
				itemTypeName = itemType.ToString();
			}
			writer.WriteAttributeString("type", itemTypeName);

			return itemInterpreter;
		}

		public static void WriteItemSettingsToXml(XmlWriter writer, Stream reader, IItemInterpreter interpreterForItem, Dictionary<Guid, IItemInterpreter> itemInterpreters)
		{
			int childItemSettingsLength = CfgToXmlUtilities.ReadInt(reader);
			byte[] childItemSettings = new byte[childItemSettingsLength];
			reader.Read(childItemSettings, 0, childItemSettingsLength);

			writer.WriteStartElement("ItemSettings");
			if (interpreterForItem != null)
			{
				interpreterForItem.WriteValueToXml(writer, childItemSettings, itemInterpreters);
			}
			else
			{
				CfgToXmlUtilities.WriteByteArrayToXml(writer, childItemSettings);
			}
			writer.WriteEndElement();//ItemSettings
		}

		/// <returns>The interpreter for this item, if any</returns>
		public static IItemInterpreter WriteItemTypeToCfg(Stream writer, XmlElement item, Dictionary<string, IItemInterpreter> itemInterpreters)
		{
			string typeName = item.GetAttribute("type");
			string interpreterTypeName = item.GetAttribute("interpreter");

			Guid type;
			IItemInterpreter itemInterpreter = null;
			if (!String.IsNullOrEmpty(interpreterTypeName))
			{
				//Find the interpreter, check that it's type name matches
				itemInterpreters.TryGetValue(typeName, out itemInterpreter);
				if (itemInterpreter == null || itemInterpreter.GetType().FullName != interpreterTypeName)
				{
					Console.WriteLine("An interpreter required to read this xml is missing: {0}\nCheck results carefully!\n");
					type = new Guid(); //Try and recover, but it probably won't help
				}
				else
				{
					type = itemInterpreter.TypeGuid;
				}
			}
			else
			{
				//No interpreter, so just use the guid
				try
				{
					type = new Guid(typeName);
				}
				catch (Exception)
				{
					Console.WriteLine("Unknown layout item type ({0})! Check results carefully", typeName);
					type = new Guid(); //Try and recover, but it probably won't help
				}
			}

			XmlToCfgUtilities.WriteGuid(writer, type);

			return itemInterpreter;
		}

		public static void WriteItemSettingsToCfg(Stream writer, XmlElement item, IItemInterpreter interpreterForItem, Dictionary<string, IItemInterpreter> itemInterpreters)
		{
			XmlElement itemSettings = (XmlElement)item.SelectSingleNode("ItemSettings");
			using (new XmlToCfgUtilities.SizeWriter(writer))
			{
				if (interpreterForItem != null)
				{
					interpreterForItem.WriteXmlToValue(writer, itemSettings, itemInterpreters);
				}
				else
				{
					XmlToCfgUtilities.WriteXmlToByteArray(writer, itemSettings);
				}
			}
		}

		/// <summary>
		/// This will read a colour from Reader and write it to Writer as 4 attributes on the current element.
		/// </summary>
		public static void WriteColorToXml(XmlWriter writer, Stream reader)
		{
			writer.WriteAttributeString("R", XmlConvert.ToString(reader.ReadByte()));
			writer.WriteAttributeString("G", XmlConvert.ToString(reader.ReadByte()));
			writer.WriteAttributeString("B", XmlConvert.ToString(reader.ReadByte()));
			writer.WriteAttributeString("A", XmlConvert.ToString(reader.ReadByte()));
		}
		/// <summary>
		/// This will take a colour provided as a byte array (length 4) and write it to Writer as 4 attributes on the current element.
		/// This overload is used for delayed writing of a value that has been previously read.
		/// </summary>
		public static void WriteColorToXml(XmlWriter writer, byte[] color)
		{
			writer.WriteAttributeString("R", XmlConvert.ToString(color[0]));
			writer.WriteAttributeString("G", XmlConvert.ToString(color[1]));
			writer.WriteAttributeString("B", XmlConvert.ToString(color[2]));
			writer.WriteAttributeString("A", XmlConvert.ToString(color[3]));
		}
		
		/// <summary>
		/// This will read a colour from 4 attributes on colorElement, and write it to Writer.
		/// </summary>
		public static void WriteColorToCfg(Stream writer, XmlElement colorElement)
		{
			writer.WriteByte(XmlConvert.ToByte(colorElement.GetAttribute("R")));
			writer.WriteByte(XmlConvert.ToByte(colorElement.GetAttribute("G")));
			writer.WriteByte(XmlConvert.ToByte(colorElement.GetAttribute("B")));
			writer.WriteByte(XmlConvert.ToByte(colorElement.GetAttribute("A")));
		}
	}
}
