using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;

namespace FooCfgToXml
{
	partial class Program
	{
		private static void ConvertXmlToCfg(FileInfo fileInfo)
		{
			Dictionary<Guid, IValueInterpreter> valueInterpreters = Utilities.LoadAddins<Guid, IValueInterpreter>(delegate(IValueInterpreter valueInterpreter) { return valueInterpreter.SettingGuid; });
			
			FileInfo output = new FileInfo(Path.ChangeExtension(Path.ChangeExtension(fileInfo.FullName, null), ".cfg"));
			Console.WriteLine("Converting \"{0}\"\n        to \"{1}\"\n", fileInfo.FullName, output.FullName);

			XmlDocument xmlDoc = new XmlDocument();
			try
			{
				xmlDoc.Load(fileInfo.FullName);
			}
			catch (Exception e)
			{
				Console.WriteLine("Could not read \"{0}\":\n{1}", fileInfo.Name, e.Message);
				return;
			}

			//Open output file
			try
			{
				if (output.Exists)
				{
					FileInfo backupFile = GetNonExistantFilename(new FileInfo(String.Format("{0}.bak", output.FullName)));
					Console.WriteLine(" Creating backup as: \"{0}\"", backupFile.Name);
					try
					{
						output.CopyTo(backupFile.FullName);
					}
					catch (Exception e)
					{
						Console.WriteLine("Aborting: could not create backup file \"{0}\":\n{1}", backupFile.FullName, e.Message);
						return;
					}

					Console.WriteLine(" Deleting existing \"{0}\"", output.Name);
					try
					{
						output.Delete();
					}
					catch (Exception e)
					{
						Console.WriteLine("Aborting: could not delete file: {0}", e.Message);
						return;
					}
				}

				using (FileStream writer = output.Create())
				{
					XmlToCfgUtilities.WriteGuid(writer, sFooConfigGuid);
					XmlToCfgUtilities.WriteInt(writer, 0); //Checksum set as 0

					XmlNodeList settingsBlocks = xmlDoc.DocumentElement.SelectNodes("Component");

					XmlToCfgUtilities.WriteInt(writer, settingsBlocks.Count); //Number of settings blocks

					foreach (XmlElement settingsBlock in settingsBlocks)
					{
						string name = settingsBlock.GetAttribute("name");
						XmlToCfgUtilities.WriteString(writer, name, true, true);
						XmlToCfgUtilities.WriteInt(writer, 0); //Write the misterious 0

						using (new XmlToCfgUtilities.SizeWriter(writer)) //Automatically fills in the block size
						{
							foreach (XmlElement setting in settingsBlock.SelectNodes("Setting"))
							{
								Guid settingGuid = new Guid(setting.GetAttribute("guid"));
								XmlToCfgUtilities.WriteGuid(writer, settingGuid);

								using (new XmlToCfgUtilities.SizeWriter(writer)) //Automatically fills in the value size
								{
									string interpreterTypeName = setting.GetAttribute("interpreter");

									IValueInterpreter valueInterpreter = null;
									if(!String.IsNullOrEmpty(interpreterTypeName))
									{
										//Find the interpreter, check that it's type name matches
										valueInterpreters.TryGetValue(settingGuid, out valueInterpreter);
										if(valueInterpreter == null || valueInterpreter.GetType().FullName != interpreterTypeName)
										{
											Console.WriteLine("An interpreter required to read this xml is missing: {0}\nThis setting will not have a value!\n");
											continue; //No value for this setting.
										}
									}
									
									if (valueInterpreter != null)
									{
										valueInterpreter.WriteXmlToValue(writer, setting);
									}
									else
									{
										XmlToCfgUtilities.WriteXmlToByteArray(writer, setting);
									}
								}
							}
						}
					}
				}
			}
			catch (Exception e)
			{
				Console.WriteLine("Could not write to \"{0}\":\n{1}", output.Name, e.Message);
				return;
			}

			Console.WriteLine(" Successfully converted!");
		}
	}
}
